;+
; @file_comments compute the latitudes of the n128 gaussian grid. See:
; http://www.ecmwf.int/products/data/technical/gaussian/n128FIS.html
;
; @categories
; Grid
;
; @examples 
; IDL> lat=n128gaussian()
;
; @returns a 1d array
;
; @history
;      Sebastien Masson (smasson\@lodyc.jussieu.fr)
;      June 2004
;
; @version $Id: n128gaussian.pro 157 2006-08-21 09:01:50Z navarro $
;
;-
;
FUNCTION n128gaussian
;
;
  compile_opt idl2, strictarrsubs
;
;          latitude   reduced     regular  latitude
;           number     points      points
;          -------    -------     -------  --------
  n128 = [    1,         18,        512,  89.46282, $
              2,         25,        512,  88.76695, $
              3,         36,        512,  88.06697, $
              4,         40,        512,  87.36606, $
              5,         45,        512,  86.66480, $
              6,         50,        512,  85.96337, $
              7,         60,        512,  85.26184, $
              8,         64,        512,  84.56026, $
              9,         72,        512,  83.85863, $
              10,         72,        512,  83.15698, $
              11,         80,        512,  82.45531, $
              12,         90,        512,  81.75363, $
              13,         90,        512,  81.05194, $
              14,        100,        512,  80.35023, $
              15,        108,        512,  79.64852, $
              16,        120,        512,  78.94681, $
              17,        120,        512,  78.24509, $
              18,        125,        512,  77.54336, $
              19,        128,        512,  76.84163, $
              20,        144,        512,  76.13990, $
              21,        144,        512,  75.43817, $
              22,        150,        512,  74.73644, $
              23,        160,        512,  74.03470, $
              24,        160,        512,  73.33296, $
              25,        180,        512,  72.63123, $
              26,        180,        512,  71.92949, $
              27,        180,        512,  71.22774, $
              28,        192,        512,  70.52600, $
              29,        192,        512,  69.82426, $
              30,        200,        512,  69.12252, $
              31,        216,        512,  68.42077, $
              32,        216,        512,  67.71903, $
              33,        216,        512,  67.01728, $
              34,        225,        512,  66.31554, $
              35,        240,        512,  65.61379, $
              36,        240,        512,  64.91204, $
              37,        240,        512,  64.21030, $
              38,        250,        512,  63.50855, $
              39,        250,        512,  62.80680, $
              40,        256,        512,  62.10505, $
              41,        270,        512,  61.40330, $
              42,        270,        512,  60.70156, $
              43,        288,        512,  59.99981, $
              44,        288,        512,  59.29806, $
              45,        288,        512,  58.59631, $
              46,        300,        512,  57.89456, $
              47,        300,        512,  57.19281, $
              48,        320,        512,  56.49106, $
              49,        320,        512,  55.78931, $
              50,        320,        512,  55.08756, $
              51,        320,        512,  54.38581, $
              52,        324,        512,  53.68406, $
              53,        360,        512,  52.98231, $
              54,        360,        512,  52.28056, $
              55,        360,        512,  51.57881, $
              56,        360,        512,  50.87705, $
              57,        360,        512,  50.17530, $
              58,        360,        512,  49.47355, $
              59,        360,        512,  48.77180, $
              60,        375,        512,  48.07005, $
              61,        375,        512,  47.36830, $
              62,        375,        512,  46.66655, $
              63,        375,        512,  45.96479, $
              64,        384,        512,  45.26304, $
              65,        384,        512,  44.56129, $
              66,        400,        512,  43.85954, $
              67,        400,        512,  43.15779, $
              68,        400,        512,  42.45604, $
              69,        400,        512,  41.75428, $
              70,        405,        512,  41.05253, $
              71,        432,        512,  40.35078, $
              72,        432,        512,  39.64903, $
              73,        432,        512,  38.94728, $
              74,        432,        512,  38.24552, $
              75,        432,        512,  37.54377, $
              76,        432,        512,  36.84202, $
              77,        432,        512,  36.14027, $
              78,        450,        512,  35.43851, $
              79,        450,        512,  34.73676, $
              80,        450,        512,  34.03501]
  n128 = [n128, $
          81,        450,        512,  33.33326, $
          82,        450,        512,  32.63150, $
          83,        480,        512,  31.92975, $
          84,        480,        512,  31.22800, $
          85,        480,        512,  30.52625, $
          86,        480,        512,  29.82449, $
          87,        480,        512,  29.12274, $
          88,        480,        512,  28.42099, $
          89,        480,        512,  27.71924, $
          90,        480,        512,  27.01748, $
          91,        480,        512,  26.31573, $
          92,        480,        512,  25.61398, $
          93,        486,        512,  24.91223, $
          94,        486,        512,  24.21047, $
          95,        486,        512,  23.50872, $
          96,        500,        512,  22.80697, $
          97,        500,        512,  22.10521, $
          98,        500,        512,  21.40346, $
          99,        500,        512,  20.70171, $
          100,        500,        512,  19.99996, $
          101,        500,        512,  19.29820, $
          102,        500,        512,  18.59645, $
          103,        512,        512,  17.89470, $
          104,        512,        512,  17.19294, $
          105,        512,        512,  16.49119, $
          106,        512,        512,  15.78944, $
          107,        512,        512,  15.08768, $
          108,        512,        512,  14.38593, $
          109,        512,        512,  13.68418, $
          110,        512,        512,  12.98243, $
          111,        512,        512,  12.28067, $
          112,        512,        512,  11.57892, $
          113,        512,        512,  10.87717, $
          114,        512,        512,  10.17541, $
          115,        512,        512,   9.47366, $
          116,        512,        512,   8.77191, $
          117,        512,        512,   8.07016, $
          118,        512,        512,   7.36840, $
          119,        512,        512,   6.66665, $
          120,        512,        512,   5.96490, $
          121,        512,        512,   5.26314, $
          122,        512,        512,   4.56139, $
          123,        512,        512,   3.85964, $
          124,        512,        512,   3.15788, $
          125,        512,        512,   2.45613, $
          126,        512,        512,   1.75438, $
          127,        512,        512,   1.05262, $
          128,        512,        512,   0.35087]
;
  n128 = reform(n128, 4, 128, /over)
  n128 = reform(n128[3, *], /over)
  n128 = [n128, -reverse(n128)]

  return, n128
end
